from cloudinit import util
from cloudinit import url_helper
from cloudinit import version

import os
import json
import time
import urllib
import tempfile
import tarfile
import base64
import signal
import errno

from cloudinit import log as logging
from distutils import version as vr
from distutils.spawn import find_executable
from hashlib import md5
from functools import wraps

LOG = logging.getLogger(__name__)

previous_version = version.version()

class go2dir:
    def __init__(self, work_dir):
        self.work_dir = work_dir

    def __enter__(self):
        util.ensure_dir(self.work_dir)
        return self.work_dir

    def __exit__(self, exec_type, exec_value, exec_tb):
        util.del_dir(self.work_dir)


class TimeoutError(Exception):
    pass


def timeout(seconds=10, error_message=os.strerror(errno.ETIME)):
    def decorator(func):
        def _handle_timeout(signum, frame):
            raise TimeoutError(error_message)
        def wrapper(*args, **kwargs):
            signal.signal(signal.SIGALRM, _handle_timeout)
            signal.alarm(seconds)
            try:
                result = func(*args, **kwargs)
            finally:
                signal.alarm(0)
            return result
        return wraps(func)(wrapper)
    return decorator


@timeout(100)
def my_subp(cmd, shell=True):
    return util.subp(cmd, shell=shell)


def check_md5(file_md5, target_file):
    try:
        out = file_md5.strip() == md5(open(target_file, 'rb').read()).hexdigest().strip()
    except:
        out = False
    finally:
        return out


def do_download(url, path):
    timeout = 5
    retries = 1

    download_start = time.time()
    rsp = url_helper.readurl(url, timeout=timeout, retries=retries)
    util.write_file(path, rsp.contents)
    return time.time() - download_start


def download_package(pkg_info, work_dir):
    status_code = 0
    desc = download_duration = ''
    pkg_url = pkg_info['download_url']
    pkg_path = os.path.join(work_dir, os.path.basename(pkg_url))

    try:
        util.ensure_dir(work_dir)
        util.del_file(pkg_path)
        download_duration = do_download(pkg_url, pkg_path)
    except Exception as e:
        status_code = 41001
        desc = str(e)
        return (status_code, desc, 0, '')

    if not check_md5(pkg_info['md5'], pkg_path):
        status_code = 42001
        desc = "md5 check failed"

    return (status_code, desc, download_duration, pkg_path)


def extract_pkg(pkg_path, target_dir):
    if not os.path.exists(pkg_path) or \
       not tarfile.is_tarfile(pkg_path) or \
       not os.path.isdir(target_dir):
        return ''

    try:
        tar = tarfile.open(pkg_path, 'r')
        tar.extractall(target_dir)
    except:
        return ''
    else:
        l = os.path.basename(pkg_path)
        if l.endswith('.tar.gz'):
            return l[:-7]
        if l.endswith('.tgz'):
            return l[:-4]


def install_package(pkg_path, work_dir, distro):
    status_code = 0
    desc = ''
    install_start = time.time()

    pkg_name = extract_pkg(pkg_path, work_dir)
    if not pkg_name:
        status_code = 51001
        desc = "extract package failed"
        return (status_code, desc, 0)

    try:
        pkg_dir = os.path.join(work_dir, pkg_name)
        deploy_script = pkg_dir + '/tools/deploy.sh'
        cmd = "bash %s %s %s" % (deploy_script, distro.name, distro.major_version)
        out = json.loads( my_subp(cmd, shell=True)[0] )
    except TimeoutError as e:
        status_code = 52002
        util.logexc(LOG, e)
        desc = str(e)
    except Exception as e:
        status_code = 52001
        util.logexc(LOG, e)
        desc = str(e)
    else:
        status_code = int(out.get('status_code'))
        if status_code != 0:
            status_code += 53000
        desc = out.get('description')


    install_duration = time.time() - install_start
    return (status_code, desc, install_duration)


def get_new_version():
    try:
        reload(version)
    except:
        return ''
    else:
        return version.version()


def report_result(url, params):
    timeout = 5
    try:
        report_url = '%s?%s' % (url, urllib.urlencode(params))
        url_helper.readurl(url=report_url, timeout=timeout)
    except:
        util.logexc(LOG, "faild to report upgrade status")
        return 1
    else:
        return int(params['status_code']) % 100


def fix_params(func):
    def _fix_params(params):
        if not isinstance(params, (dict,)):
            return params
        global previous_version
        std_params = {
                'status_code': 0,
                'description': '',
                'download_duration': 0,
                'install_duration': 0,
                'previous_version': str(previous_version),
                'current_version': str(get_new_version()),
        }
        std_params.update(params)
        std_params['description'] = base64.b64encode(std_params['description'])
        std_params['install_duration'] = int(1000 * std_params['install_duration'])
        std_params['download_duration'] = int(1000 * std_params['download_duration'])
        return std_params

    def inner_func(*args, **kwargs):
        return _fix_params( func(*args, **kwargs) )
    return inner_func


@fix_params
def do_upgrade(cloud, log, pkg_info):

    cur_version = version.version()
    new_version = vr.StrictVersion(pkg_info['version'])
    if cur_version == new_version:
        desc = "current version %s is the newest, do nothing" % cur_version
        log.debug(desc)
        return {"status_code": 20000, "description": desc}

    with go2dir(tempfile.mkdtemp()) as work_dir:

        status_code, desc, download_duration, pkg_path = download_package(pkg_info, work_dir)
        if status_code != 0:
            log.warn("fetch cloud-init package error")
            return {'status_code': status_code,
                    'description': desc,
                    'download_duration': download_duration}

        status_code, desc, install_duration = install_package(pkg_path, work_dir, cloud.distro)
        if status_code != 0:
            log.warn("faild to install cloud-init package, status_code %s" % status_code)
        else:
            status_code = 20000
            desc = "ok"

        return {'status_code': status_code,
                'description': desc,
                'download_duration': download_duration,
                'install_duration': install_duration}



def upgrade_handler(name, cfg, cloud, log, _args):
    version_url = cloud.datasource.version_url
    report_url = cloud.datasource.report_url
    timeout = 5

    if not cloud.distro.major_version:
        log.warn("faild to get distro major version")
        return 1

    if not url_helper.wait_for_url(urls=[version_url], timeout=timeout):
        log.warn("the %s is not response, upgrade not work" % version_url)
        return 2

    try:
        pkg_info = json.loads( str(url_helper.readurl(url=version_url, timeout=timeout)) )
        params = do_upgrade(cloud, log, pkg_info)
    except Exception as e:
        params = {  'status_code': 30001,
                    'description': base64.b64encode(str(e)) }
    finally:
        return report_result(report_url, params)


def check_virt_what_local():
    for i in ('virt-what-cpuid-helper', 'virt-what'):
        if not find_executable(i):
            continue
        try:
            out = util.subp([i])[0].strip()
        except:
            pass
        else:
            if 'kvm' in out.lower():
                return 'kvm'
            elif 'xen' in out.lower():
                return 'xen'
            else:
                return out
    return ''


def check_virt_what_net():
    return 'unknow'


def check_virt_what():
    virt_what = check_virt_what_local()
    if not virt_what:
        virt_what = check_virt_what_net()
    return virt_what.strip()
