from cloudinit import log as logging
from cloudinit.settings import PER_INSTANCE
from cloudinit import templater
from cloudinit import util

from distutils.spawn import find_executable
import os
import glob
import socket
import struct

LOG = logging.getLogger(__name__)

frequency = PER_INSTANCE


def cidr(prefix):
    return socket.inet_ntoa(struct.pack(">I", (0xffffffff << (32 - prefix)) & 0xffffffff))


def add_ip_cmd(ip, mask, dev):
    if find_executable('ifconfig'):
        cmd = 'ifconfig {dev} {ip}/{mask}'.format(dev=dev, ip=ip, mask=mask)
    elif find_executable('ip'):
        cmd = 'ip addr add {ip}/{mask} dev {dev}'.format(dev=dev, ip=ip, mask=mask)
    else:
        return ('', False)

    try:
        out = util.subp(cmd, shell=True)
    except:
        out = ('', False)
    finally:
        return out


def calc_ip_segment(ip, mask):
    count = 2**(32 - int(mask)) - 2
    out = []
    part0, part1, part2, part3 = (int(i) for i in ip.split('.') )
    idx = 0
    while idx < count:
        idx += 1
        part3 += 1
        if part3 > 255:
            part3 = 0
            part2 += 1
            if part2 > 255:
                part2 = 0
                part1 += 1
                if part1 > 255:
                    part1 = 0
                    part0 += 1
        yield '.'.join( [str(i) for i in (part0, part1, part2, part3)] )


def clean_lo():
    if find_executable('ip'):
        get_ip_cmd = "ip a show dev lo | awk '/inet .*global .*lo.*/ {print $2\" \"$NF}'"
        try:
            out = util.subp(get_ip_cmd, shell=True)[0].split('\n')
        except:
            return
        for i in sorted(out,
                        key=lambda x: '' if not x.split() else x.split()[-1],
                        reverse=True):
            if not i.strip() or '/' not in i: continue
            (ip, dev) = i.strip().split()
            del_ip_cmd = "ip a del %s dev %s" % (ip, dev)
            try:
                util.subp(del_ip_cmd, shell=True)
            except:
                pass

    elif find_executable('ifconfig'):
        get_if_cmd = "ifconfig -a | awk '/^lo:[[:digit:]]+/ {print $1}'"
        try:
            out = util.subp(get_if_cmd, shell=True)
        except:
            return
        for i in sorted(out[0].split('\n')):
            if not i.strip(): continue
            del_if_cmd = "ifconfig %s down" % i.strip()
            try:
                util.subp(del_if_cmd, shell=True)
            except:
                pass


def conf_redhat_ip_segment(name, cfg, _cloud, log, _args):
    (ip, mask) = cfg['ip_segment_conf'].split()
    dev = 'lo'

    for ip_file in glob.glob('/etc/sysconfig/network-scripts/ifcfg-lo:*'):
        os.remove(ip_file)

    idx = 0
    for seg_ip in calc_ip_segment(ip, mask):
        dev = 'lo:%s' % idx
        add_ip_cmd(seg_ip, mask, dev)
        ifcfg_content = '\n'.join( ("%s=%s" % (k, v) for (k, v) in
                          { 'DEVICE': dev,
                            'IPADDR': seg_ip,
                            'NETMASK': cidr(int(mask)) }.items()) )
        ifcfg_fn = '/etc/sysconfig/network-scripts/ifcfg-%s' % dev
        util.write_file(ifcfg_fn, ifcfg_content)
        idx += 1


def conf_debian_ip_segment(name, cfg, _cloud, log, _args):
    (ip, mask) = cfg['ip_segment_conf'].split()
    dev = 'lo'

    ifn = '/etc/network/interfaces'
    try:
        if_contents = '\n'.join( [i.strip() for i in util.load_file(ifn).split('\n') if not i.strip().startswith('#')] )
    except Exception:
        if_contents = ''

    out = []
    for i in if_contents.split('auto'):
        if not i.strip().startswith('lo:'):
            out.append( 'auto ' + i.lstrip() )

    idx = 0
    for seg_ip in calc_ip_segment(ip, mask):
        dev = "lo:%s" % idx
        add_ip_cmd(seg_ip, mask,  dev)
        out.append( '\n' +
                    '\n'.join(("auto %s" % dev,
                               "iface %s inet static" % dev,
                               "address %s" % seg_ip,
                               "netmask %s" % cidr(int(mask)) ))
                    +'\n')
        idx += 1
    util.write_file(ifn, ''.join(out))


def conf_suse_ip_segment(name, cfg, _cloud, log, _args):
    (ip, mask) = cfg['ip_segment_conf'].split()
    dev = 'lo'

    for ip_file in glob.glob('/etc/sysconfig/network/ifcfg-lo:*'):
        os.remove(ip_file)

    idx = 0
    for seg_ip in calc_ip_segment(ip, mask):
        dev = 'lo:%s' % idx
        add_ip_cmd(seg_ip, mask,  dev)
        ifcfg_content = '\n'.join( ("%s_%s=%s" % (k, idx, v) for (k, v) in
                          { 'IPADDR': seg_ip,
                            'NETMASK': cidr(int(mask)),
                            'LABEL': idx}.items()) )
        ifcfg_fn = '/etc/sysconfig/network/ifcfg-%s' % dev
        util.write_file(ifcfg_fn, ifcfg_content)
        idx += 1


def conf_gentoo_ip_segment(name, cfg, _cloud, log, _args):
    pass


def handle(name, cfg, _cloud, log, _args):
    if not cfg.get('ip_segment_conf'):
        log.debug(("Skipping module named %s,"
                   " no 'ip_segment_conf' key in configuration"), name)
        return

    _distro_name = _cloud.distro.name
    if _distro_name in ('rhel', 'centos'):
        functor = conf_redhat_ip_segment
    elif _distro_name in ('ubuntu', 'debian'):
        functor = conf_debian_ip_segment
    elif _distro_name in ('sles','opensuse', ):
        functor = conf_suse_ip_segment
    elif _distro_name in ('gentoo', ):
        functor = conf_gentoo_ip_segment
    else:
        log.debug("not support distribution %s" % _distro_name)
        return
    clean_lo()
    functor(name, cfg, _cloud, log, _args)
