# vi: ts=4 expandtab

import os

from cloudinit import log as logging
from cloudinit import aliyun_utils
from cloudinit import sources
from cloudinit.sources import DataSourceEc2 as EC2

LOG = logging.getLogger(__name__)

DEF_MD_VERSION = "2016-01-01"


class DataSourceAliYun(EC2.DataSourceEc2):
    metadata_urls = ["http://100.100.100.200"]

    def __init__(self, sys_cfg, distro, paths):
        super(DataSourceAliYun, self).__init__(sys_cfg, distro, paths)
        self.seed_dir = os.path.join(paths.seed_dir, "AliYun")
        self.api_ver = DEF_MD_VERSION
        self.support_upgrade_cloudinit = True
        self.upgrade_handler = aliyun_utils.upgrade_handler
        self.version_url_suffix = '/cloudinit/version'
        self.report_url_suffix = '/cloudinit/report'
        self._support_xen = False
        self._virt_what = None

    @property
    def virt_what(self):
        if not self._virt_what:
            self._virt_what = aliyun_utils.check_virt_what()
        return self._virt_what

    def check_support_xen_net(self):
        return None

    @property
    def support_xen(self):
        md_support_xen = self.check_support_xen_net()
        if md_support_xen is not None:
            return md_support_xen
        else:
            if not self.ds_cfg:
                return False
            else:
                return self.ds_cfg.get('support_xen', False)

    def get_data(self):
        if self.virt_what == 'xen' and not self.support_xen:
            LOG.debug("the virtualization platform is xen, not support now")
            return False
        return super(DataSourceAliYun, self).get_data()

    def get_hostname(self, fqdn=False, _resolve_ip=False):
        return self.metadata.get('hostname', 'localhost.localdomain')

    def get_public_ssh_keys(self):
        return parse_public_keys(self.metadata.get('public-keys', {}))

    def get_ntp_conf(self):
        if not self.metadata:
            return {}
        return self.metadata.get('ntp-conf')

    def get_source_address(self):
        if not self.metadata or not self.metadata.get('source-address'):
            return []
        source_address = self.metadata.get('source-address')
        if isinstance(source_address, str):
            source_address = [source_address]
        return source_address

    @property
    def version_url(self):
        return self.metadata_address + self.version_url_suffix

    @property
    def report_url(self):
        return self.metadata_address + self.report_url_suffix


def parse_public_keys(public_keys):
    keys = []
    for key_id, key_body in public_keys.items():
        if isinstance(key_body, str):
            keys.append(key_body.strip())
        elif isinstance(key_body, list):
            keys.extend(key_body)
        elif isinstance(key_body, dict):
            key = key_body.get('openssh-key', [])
            if isinstance(key, str):
                keys.append(key.strip())
            elif isinstance(key, list):
                keys.extend(key)
    return keys

# Used to match classes to dependencies
datasources = [
    (DataSourceAliYun, (sources.DEP_FILESYSTEM, sources.DEP_NETWORK)),
]


# Return a list of data sources that match this set of dependencies
def get_datasource_list(depends):
    return sources.list_from_depends(depends, datasources)
