#!/bin/bash


map=(
aliyun:6:sysvinit
aliyun:1501:sysvinit
aliyun:1903:systemd
aliyun:7:systemd
aliyun:1701:systemd
centos:5:sysvinit
centos:6:sysvinit
centos:7:systemd
rhel:5:sysvinit
rhel:6:sysvinit
rhel:7:systemd
redhat:5:sysvinit
redhat:6:sysvinit
redhat:7:systemd
debian:6:sysvinit_deb
debian:7:sysvinit_deb
debian:8:systemd
debian:9:systemd
ubuntu:12:upstart
ubuntu:14:upstart
ubuntu:16:systemd
ubuntu:18:systemd
opensuse:13:systemd
opensuse:1301:systemd
opensuse:1302:systemd
opensuse:42:systemd
opensuse:4201:systemd
opensuse:4202:systemd
opensuse:4203:systemd
sles:11:sysvinit
sles:1101:sysvinit
sles:1102:sysvinit
sles:1103:sysvinit
sles:1104:sysvinit
sles:12:systemd
sles:1201:systemd
sles:1202:systemd
sles:1203:systemd
)


my_echo() {
	echo "$@" >> $deploy_log
}


my_date_echo() {
	my_echo "[$(date)] $@"
}


line_barrier() {
	my_date_echo "=============== $@ ==============="
}


my_json_echo() {
	echo -e "{\n\
\"status_code\": $1, \n\
\"description\": \"$2\"\n\
}"
}


my_exec() {
	local time_out=$1
	shift
	local cmd="$@"
	if has timeout; then
		timeout $time_out bash -c "$cmd" >> $deploy_log 2>&1
	else
		$cmd >> $deploy_log 2>&1
	fi
	o=$?
	if [[ $o -eq 124 ]]; then
		my_exit 9 "execute cmd: '$cmd' timeout $time_out"
	else
		return $o
	fi
}


my_exit() {
	my_echo "******** $2 ********"
	my_json_echo "$1" "$2"
	exit 0
}


my_warn() {
	my_echo "**WARN: $1, $2**"
}


get_initsystem() {
	local issue=$1
	local big=$2
	for i in ${map[@]}; do
		if echo $i | grep -E -q "$issue:$big:" ; then
			echo $i | awk -F':' '{print $3}'
		fi
	done
}


has() {
	which $1 >& /dev/null
}


check_remote_host() {
	local target=$1
	local port=$2
	local tm=1

	if has nc ; then
		nc -z -w $tm $target $port >& /dev/null
	else
		timeout $tm bash -c "cat < /dev/null > /dev/tcp/$target/$port"
	fi
}


get_pypi_source() {
pypi_source_list=(
mirrors.cloud.aliyuncs.com
mirrors.aliyun.com
)
	for i in ${pypi_source_list[@]}; do
		if check_remote_host $i 80; then
			echo $i
			return 0
		fi
	done
	return 1
}


fix_pypi_source() {
	local pypi_source=$1

	if [[ ! -f ~/.pydistutils.cfg ]]; then
cat > ~/.pydistutils.cfg << EOF
[easy_install]
index-url=http://$pypi_source/pypi/simple/
EOF
	else
		sed -i "s#index-url.*#index-url=http://$pypi_source/pypi/simple/#" ~/.pydistutils.cfg
	fi

	if [[ ! -f ~/.pip/pip.conf ]]; then
	mkdir -p ~/.pip
cat > ~/.pip/pip.conf << EOF
[global]
index-url=http://$pypi_source/pypi/simple/

[install]
trusted-host=$pypi_source
EOF
	else
		sed -i "s#index-url.*#index-url=http://$pypi_source/pypi/simple/#" ~/.pip/pip.conf
		sed -i "s#trusted-host.*#trusted-host=$pypi_source#" ~/.pip/pip.conf
	fi
}


install_deps() {
	if ! has pip ; then
		my_echo "no pip, do nothing"
		return
	fi
	[[ -z "$pip_big_version" ]] && return

	if [[ -n "$pypi_source" ]]; then
		opts="-i http://$pypi_source/pypi/simple"
		[[ "$pip_big_version" -ge 7 ]] && opts=${opts}" --trusted-host $pypi_source"
	fi

	local cmd="pip install $opts -r $CLOUDINIT_DIR/requirements.txt"
	my_date_echo "cmd: $cmd"
	my_exec $cmd_timeout $cmd || my_warn 10 'handle deps faild' # but not block the installation
}


install_cloudinit() {
	cd $CLOUDINIT_DIR
	local cmd="python setup.py install --init-system=$init_system"
	my_date_echo "cmd: $cmd"

	my_exec $cmd_timeout $cmd || my_exit 11 'install faild'

	! has cloud-init && my_exit 12 'no has cloud-init'

	cloud_init_std_path=/usr/bin/cloud-init
	fn=$(which cloud-init)
	if has realpath; then
		fn=$(realpath $fn)
	fi
	if [[ "$fn" != "$cloud_init_std_path" ]]; then
		mv $cloud_init_std_path ${cloud_init_std_path}.old >& /dev/null
		ln -s $fn $cloud_init_std_path
	fi
}


install_service() {
	local cmd=''

	# ensure service file
	if [[ "$init_system" == 'sysvinit' ]]; then
		cp $CLOUDINIT_DIR/sysvinit/redhat/* /etc/init.d/
	fi

	for i in 'cloud-init-local' 'cloud-init' 'cloud-config' 'cloud-final' ; do
		if  [[ "$init_system" == 'sysvinit' ]]; then
			cmd="chkconfig --add $i && chkconfig $i on"
		elif [[ "$init_system" == 'systemd' ]]; then
			cmd="systemctl enable ${i}.service"
		elif [[ "$init_system" == 'sysvinit_deb' ]]; then
			cmd="update-rc.d $i defaults"
		fi

		my_echo "cmd: $cmd"
		bash -c "$cmd" >> $deploy_log 2>&1 || my_exit 8 "install service failed: cmd:'$cmd', issue:$issue, version:$version, init_system:$init_system"
	done
}


prepare_env() {
	[[ $(id -u) != 0 ]] && my_json_echo 1 "permission denied, you are not root" && exit

	deploy_log=/var/log/cloudinit-deploy.log
	> $deploy_log
	line_barrier "prepare_env"

	CLOUDINIT_DIR=$(readlink -f $(dirname $0))/../
	[[ -z $CLOUDINIT_DIR ]] && my_exit 1 'no work dir'
	[[ -n $1 ]] && issue=$1 || my_exit 1 'no os issue'
	[[ -n $2 ]] && version=$2 || my_exit 2 'no os version'

	init_system=$(get_initsystem $issue $version)
	[[ -z $init_system ]] && my_exit 3 "no init_system for $issue $version"

	if has python ; then
		py_version=$(python -V 2>&1 | awk '{print $NF}')
	else
		my_exit 4 'no python'
	fi

	if ! python -c 'import setuptools' >& /dev/null; then
		my_exit 4 'no setuptools module in python'
	fi

	if has pip ; then
		pip_version=$(pip --version | awk '{print $2}')
		pip_big_version=$(echo $pip_version | awk -F'.' '{print $1}')
		[[ -z $pip_big_version ]] && my_warn 5 'no pip version'
	else
		my_warn 5 'no pip'
	fi

	pypi_source=$(get_pypi_source)
	if [[ -n $pypi_source ]]; then
		fix_pypi_source $pypi_source || my_warn 7 'faild fix pypi source'
	else
		my_warn 6 'no pypi source'
	fi

	cmd_timeout='60s'

	my_echo "os issue: $issue"
	my_echo "os version: $version"
	my_echo "init system: $init_system"
	my_echo "cloudinit direcotry: $CLOUDINIT_DIR"
	my_echo "python version: $py_version"
	my_echo "pip version: $pip_version"
	my_echo "pypi source: $pypi_source"
	my_echo "deploy log: $deploy_log"
	my_echo "execute cmd timeout: $cmd_timeout"
}


post_job() {
	cfg_file=$CLOUDINIT_DIR/cfg/${issue}_cloud.cfg
	cp $cfg_file /etc/cloud/cloud.cfg
}


do_deploy() {
	prepare_env "$@"

	line_barrier "install_deps"
	install_deps

	line_barrier "install_cloudinit"
	install_cloudinit

	line_barrier "install_service"
	install_service

	line_barrier "post_job"
	post_job

	my_exit 0 "success"
}


do_deploy $1 $2


# status:


# Success:
# 0, success

# Error:
#  if happend, stop the installation and print to stdout as json format
# 1. no su
# 1, no work dir
# 1, no os issue
# 2, no os version
# 3, no init system
# 4, no python
# 4, no setuptools module in python
# 8, install service failed
# 11, install failed
# 12, installed but not cloud-init executabled

# Warn:
#  record in log file, not block the procedure.
# 5, no pip
# 6, no pipy source
# 7, faild fix pypi source
# 10, handle deps failed
